@@ -50,8 +50,8 @@
 
 struct _MetaWaylandKeyboardGrabInterface
 {
-  void (*key) (MetaWaylandKeyboardGrab * grab, uint32_t time,
-               uint32_t key, uint32_t state);
+  gboolean (*key) (MetaWaylandKeyboardGrab * grab, uint32_t time,
+		   uint32_t key, uint32_t state);
   void (*modifiers) (MetaWaylandKeyboardGrab * grab, uint32_t serial,
                      uint32_t mods_depressed, uint32_t mods_latched,
                      uint32_t mods_locked, uint32_t group);
@@ -96,7 +96,6 @@ struct _MetaWaylandKeyboard
   struct wl_resource *focus_resource;
   struct wl_listener focus_listener;
   uint32_t focus_serial;
-  struct wl_signal focus_signal;
 
   MetaWaylandKeyboardGrab *grab;
   MetaWaylandKeyboardGrab default_grab;
@@ -111,9 +110,7 @@ struct _MetaWaylandKeyboard
   struct wl_display *display;
 
   struct xkb_context *xkb_context;
-  struct xkb_state *xkb_state;
   gboolean is_evdev;
-
   MetaWaylandXkbInfo xkb_info;
   struct xkb_rule_names xkb_names;
 
@@ -126,7 +123,7 @@ meta_wayland_keyboard_init (MetaWaylandKeyboard *keyboard,
                             struct wl_display   *display,
 			    gboolean             is_evdev);
 
-void
+gboolean
 meta_wayland_keyboard_handle_event (MetaWaylandKeyboard *keyboard,
                                     const ClutterKeyEvent *event);
 
