@@ -32,6 +32,8 @@
 #include <sys/stat.h>
 #include <stdlib.h>
 #include <sys/wait.h>
+#include <fcntl.h>
+#include <unistd.h>
 
 #include <wayland-server.h>
 
@@ -185,6 +187,7 @@ meta_wayland_buffer_reference (MetaWaylandBufferReference *ref,
 
       if (ref->buffer->busy_count == 0)
         {
+	  g_clear_pointer (&ref->buffer->texture, cogl_object_unref);
           g_assert (wl_resource_get_client (ref->buffer->resource));
           wl_resource_queue_event (ref->buffer->resource, WL_BUFFER_RELEASE);
         }
@@ -216,9 +219,7 @@ meta_wayland_compositor_set_input_focus (MetaWaylandCompositor *compositor,
 void
 meta_wayland_compositor_repick (MetaWaylandCompositor *compositor)
 {
-  meta_wayland_seat_repick (compositor->seat,
-                            get_time (),
-                            NULL);
+  meta_wayland_seat_repick (compositor->seat, NULL);
 }
 
 static void
@@ -230,11 +231,10 @@ meta_wayland_compositor_create_surface (struct wl_client *wayland_client,
     wl_resource_get_user_data (wayland_compositor_resource);
   MetaWaylandSurface *surface;
 
-  /* a surface inherits the version from the compositor */
   surface = meta_wayland_surface_create (compositor,
-					 wayland_client,
-					 id,
-					 wl_resource_get_version (wayland_compositor_resource));
+					 wayland_client, id,
+					 MIN (META_WL_SURFACE_VERSION,
+					      wl_resource_get_version (wayland_compositor_resource)));
   
   compositor->surfaces = g_list_prepend (compositor->surfaces, surface);
 }
@@ -297,7 +297,9 @@ meta_wayland_compositor_create_region (struct wl_client *wayland_client,
   MetaWaylandRegion *region = g_slice_new0 (MetaWaylandRegion);
 
   region->resource = wl_resource_create (wayland_client,
-					 &wl_region_interface, 1,
+					 &wl_region_interface,
+					 MIN (META_WL_REGION_VERSION,
+					      wl_resource_get_version (compositor_resource)),
 					 id);
   wl_resource_set_implementation (region->resource,
 				  &meta_wayland_region_interface, region,
@@ -335,7 +337,8 @@ bind_output (struct wl_client *client,
   struct wl_resource *resource;
   guint mode_flags;
 
-  resource = wl_resource_create (client, &wl_output_interface, version, id);
+  resource = wl_resource_create (client, &wl_output_interface,
+				 MIN (META_WL_OUTPUT_VERSION, version), id);
   wayland_output->resources = g_list_prepend (wayland_output->resources, resource);
 
   wl_resource_set_user_data (resource, wayland_output);
@@ -373,7 +376,7 @@ bind_output (struct wl_client *client,
                           (int)output->crtc->current_mode->height,
                           (int)output->crtc->current_mode->refresh_rate);
 
-  if (version >= 2)
+  if (version >= META_WL_OUTPUT_HAS_DONE)
     wl_resource_post_event (resource,
                             WL_OUTPUT_DONE);
 }
@@ -476,7 +479,8 @@ meta_wayland_compositor_update_outputs (MetaWaylandCompositor *compositor,
         {
           wayland_output = g_slice_new0 (MetaWaylandOutput);
           wayland_output->global = wl_global_create (compositor->wayland_display,
-                                                     &wl_output_interface, 2,
+                                                     &wl_output_interface,
+						     META_WL_OUTPUT_VERSION,
                                                      wayland_output, bind_output);
         }
 
@@ -518,7 +522,8 @@ compositor_bind (struct wl_client *client,
   MetaWaylandCompositor *compositor = data;
   struct wl_resource *resource;
 
-  resource = wl_resource_create (client, &wl_compositor_interface, version, id);
+  resource = wl_resource_create (client, &wl_compositor_interface,
+				 MIN (META_WL_COMPOSITOR_VERSION, version), id);
   wl_resource_set_implementation (resource, &meta_wayland_compositor_interface, compositor, NULL);
 }
 
@@ -545,7 +550,7 @@ synthesize_motion_event (MetaWaylandCompositor *compositor,
   XIDeviceEvent device_event;
   unsigned char button_mask[(N_BUTTONS + 7) / 8] = { 0 };
   MetaDisplay *display = meta_get_display ();
-  ClutterModifierType state;
+  ClutterModifierType button_state;
   int i;
 
   generic_event.type = GenericEvent;
@@ -607,8 +612,8 @@ synthesize_motion_event (MetaWaylandCompositor *compositor,
       device_event.event_y = wl_fixed_to_double (pointer->y);
     }
 
-  if (surface && surface->xid != None)
-    device_event.event = surface->xid;
+  if (surface && surface->window != NULL)
+    device_event.event = surface->window->xwindow;
   else
     device_event.event = device_event.root;
 
@@ -618,10 +623,18 @@ synthesize_motion_event (MetaWaylandCompositor *compositor,
   device_event.root_x = wl_fixed_to_double (pointer->x);
   device_event.root_y = wl_fixed_to_double (pointer->y);
 
-  state = clutter_event_get_state (event);
+  clutter_event_get_state_full (event,
+				&button_state,
+				(ClutterModifierType*)&device_event.mods.base,
+				(ClutterModifierType*)&device_event.mods.latched,
+				(ClutterModifierType*)&device_event.mods.locked,
+				(ClutterModifierType*)&device_event.mods.effective);
+  device_event.mods.effective &= ~button_state;
+  memset (&device_event.group, 0, sizeof (device_event.group));
+  device_event.group.effective = (device_event.mods.effective >> 13) & 0x3;
 
   for (i = 0; i < N_BUTTONS; i++)
-    if ((state & (CLUTTER_BUTTON1_MASK << i)))
+    if ((button_state & (CLUTTER_BUTTON1_MASK << i)))
       XISetMask (button_mask, i + 1);
   device_event.buttons.mask_len = N_BUTTONS + 1;
   device_event.buttons.mask = button_mask;
@@ -630,14 +643,6 @@ synthesize_motion_event (MetaWaylandCompositor *compositor,
   device_event.valuators.mask = NULL;
   device_event.valuators.values = NULL;
 
-  memset (&device_event.mods, 0, sizeof (device_event.mods));
-  device_event.mods.effective =
-    state & (CLUTTER_MODIFIER_MASK &
-             ~(((CLUTTER_BUTTON1_MASK << N_BUTTONS) - 1) ^
-               (CLUTTER_BUTTON1_MASK - 1)));
-
-  memset (&device_event.group, 0, sizeof (device_event.group));
-
   meta_display_handle_event (display, (XEvent *) &generic_event);
 }
 
@@ -678,7 +683,8 @@ event_cb (ClutterActor *stage,
 
   reset_idletimes (event);
 
-  meta_wayland_seat_handle_event (compositor->seat, event);
+  if (meta_wayland_seat_handle_event (compositor->seat, event))
+    return TRUE;
 
   /* HACK: for now, the surfaces from Wayland clients aren't
      integrated into Mutter's event handling and Mutter won't give them
@@ -800,6 +806,35 @@ on_monitors_changed (MetaMonitorManager    *monitors,
   compositor->outputs = meta_wayland_compositor_update_outputs (compositor, monitors);
 }
 
+static void
+set_gnome_env (const char *name,
+	       const char *value)
+{
+  GDBusConnection *session_bus;
+  GError *error;
+
+  setenv (name, value, TRUE);
+
+  session_bus = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
+  g_assert (session_bus);
+
+  error = NULL;
+  g_dbus_connection_call_sync (session_bus,
+			       "org.gnome.SessionManager",
+			       "/org/gnome/SessionManager",
+			       "org.gnome.SessionManager",
+			       "Setenv",
+			       g_variant_new ("(ss)", name, value),
+			       NULL,
+			       G_DBUS_CALL_FLAGS_NONE,
+			       -1, NULL, &error);
+  if (error)
+    {
+      meta_warning ("Failed to set environment variable %s for gnome-session: %s\n", name, error->message);
+      g_clear_error (&error);
+    }
+}
+
 void
 meta_wayland_init (void)
 {
@@ -809,6 +844,7 @@ meta_wayland_init (void)
   ClutterBackend *backend;
   CoglContext *cogl_context;
   CoglRenderer *cogl_renderer;
+  char *display_name;
 
   memset (compositor, 0, sizeof (MetaWaylandCompositor));
 
@@ -821,7 +857,8 @@ meta_wayland_init (void)
   wl_list_init (&compositor->frame_callbacks);
 
   if (!wl_global_create (compositor->wayland_display,
-			 &wl_compositor_interface, 3,
+			 &wl_compositor_interface,
+			 META_WL_COMPOSITOR_VERSION,
 			 compositor, compositor_bind))
     g_error ("Failed to register wayland compositor object");
 
@@ -910,7 +947,9 @@ meta_wayland_init (void)
 
   clutter_actor_show (compositor->stage);
 
-  if (wl_display_add_socket (compositor->wayland_display, "wayland-0"))
+  /* FIXME: find the first free name instead */
+  compositor->display_name = g_strdup ("wayland-0");
+  if (wl_display_add_socket (compositor->wayland_display, compositor->display_name))
     g_error ("Failed to create socket");
 
   /* XXX: It's important that we only try and start xwayland after we
@@ -926,7 +965,11 @@ meta_wayland_init (void)
   if (!meta_xwayland_start (compositor))
     g_error ("Failed to start X Wayland");
 
-  putenv (g_strdup_printf ("DISPLAY=:%d", compositor->xwayland_display_index));
+  display_name = g_strdup_printf (":%d", compositor->xwayland_display_index);
+  set_gnome_env ("DISPLAY", display_name);
+  g_free (display_name);
+
+  set_gnome_env ("WAYLAND_DISPLAY", compositor->display_name);
 }
 
 void
@@ -945,3 +988,9 @@ meta_wayland_compositor_get_launcher (MetaWaylandCompositor *compositor)
 {
   return compositor->launcher;
 }
+
+gboolean
+meta_wayland_compositor_is_native (MetaWaylandCompositor *compositor)
+{
+  return compositor->drm_fd >= 0;
+}
